class InferenceError(Exception):
    ...

def infer_shapes(b: bytes, check_type: bool, strict_mode: bool, data_prop: bool) -> bytes: ...

def infer_shapes_path(model_path: str, output_path: str, check_type: bool, strict_mode: bool, data_prop: bool) -> None: ...
